{
  POP3 Mail Checker

  Message box routines

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit MC_MsgBox;

interface

uses Windows, Messages, SysUtils, CommDlg, Classes, Graphics, Controls,
  Forms, StdCtrls, ExtCtrls, Math;

function CreateAppMessageBox(const AText, ACaption: string; Flags: Longint): TForm;

function AppMessageBox(const Text, Caption: string; Flags: Longint): Integer;
function AppMessageBoxFmt(const TextFormat, Caption: string; Flags: Longint; const Args: array of const): Integer;

implementation

uses
  Consts;

function Max(I, J: Integer): Integer;
begin
  if I > J then Result := I else Result := J;
end;

function GetAveCharSize(Canvas: TCanvas): TPoint;
var
  I: Integer;
  Buffer: array[0..51] of Char;
begin
  for I := 0 to 25 do Buffer[I] := Chr(I + Ord('A'));
  for I := 0 to 25 do Buffer[I + 26] := Chr(I + Ord('a'));
  GetTextExtentPoint(Canvas.Handle, Buffer, 52, TSize(Result));
  Result.X := Result.X div 52;
end;

type
  TAppMsgBtn = (mbYes, mbNo, mbOK, mbCancel, mbAbort, mbRetry, mbIgnore);
  TAppMsgButtons = set of TAppMsgBtn;

const
  ButtonNames: array[TAppMsgBtn] of string = (
    'Yes', 'No', 'OK', 'Cancel', 'Abort', 'Retry', 'Ignore');
  ButtonCaptions: array[TAppMsgBtn] of Pointer = (
    @SMsgDlgYes, @SMsgDlgNo, @SMsgDlgOK, @SMsgDlgCancel, @SMsgDlgAbort,
    @SMsgDlgRetry, @SMsgDlgIgnore);
  ButtonWidths: array[TAppMsgBtn] of Integer = (
    0, 0, 0, 0, 0, 0, 0);
  ModalResults: array[TAppMsgBtn] of Integer = (
    mrYes, mrNo, mrOk, mrCancel, mrAbort, mrRetry, mrIgnore);

function CreateAppMessageBox(const AText, ACaption: string; Flags: Longint): TForm;
const
  mcHorzMargin = 8;
  mcVertMargin = 8;
  mcHorzSpacing = 10;
  mcVertSpacing = 10;
  mcButtonWidth = 50;
  mcButtonHeight = 14;
  mcButtonSpacing = 4;
  mcMaxWidthInc = 15;
var
  Button: TButton;
  Buttons: TAppMsgButtons;
  DialogUnits: TPoint;
  HorzMargin, VertMargin, HorzSpacing, VertSpacing, MaxWidthInc, ButtonWidth,
  ButtonHeight, ButtonSpacing, ButtonCount, ButtonGroupWidth,
  IconTextWidth, IconTextHeight, X, ALeft: Integer;
  B: TAppMsgBtn;
  IconID: PChar;
  TextRect: TRect;
begin
  Result := TForm.CreateNew(Application);
  with Result do
  begin
    BorderStyle := bsDialog;
    Canvas.Font := Font;
    DialogUnits := GetAveCharSize(Canvas);
    { Dialog units }
    HorzMargin := MulDiv(mcHorzMargin, DialogUnits.X, 4);
    VertMargin := MulDiv(mcVertMargin, DialogUnits.Y, 8);
    HorzSpacing := MulDiv(mcHorzSpacing, DialogUnits.X, 4);
    VertSpacing := MulDiv(mcVertSpacing, DialogUnits.Y, 8);
    MaxWidthInc := MulDiv(mcMaxWidthInc, DialogUnits.Y, 8);
    { Create buttons set }
    case Flags and $0000000F of
      MB_OKCANCEL: Buttons := [mbOK, mbCancel];
      MB_ABORTRETRYIGNORE: Buttons := [mbAbort, mbRetry, mbIgnore];
      MB_YESNOCANCEL: Buttons := [mbYes, mbNo, mbCancel];
      MB_YESNO: Buttons := [mbYes, mbNo];
      MB_RETRYCANCEL: Buttons := [mbRetry, mbCancel];
    else
      Buttons := [mbOK];
    end;
    { Button units }
    ButtonWidth := MulDiv(mcButtonWidth, DialogUnits.X, 4) - 4;
    for B := Low(TAppMsgBtn) to High(TAppMsgBtn) do
    begin
      if not (B in Buttons) then Continue;
      { Update button width value }
      if ButtonWidths[B] = 0 then
      begin
        TextRect := Rect(0,0,0,0);
        Windows.DrawText(Canvas.handle,
          PChar(LoadResString(ButtonCaptions[B])), -1,
          TextRect, DT_CALCRECT or DT_LEFT or DT_SINGLELINE);
        with TextRect do
          ButtonWidths[B] := Right - Left + 4;
      end;
      { Update max button width }
      if ButtonWidths[B] > ButtonWidth then
        ButtonWidth := ButtonWidths[B];
    end;
    ButtonHeight := MulDiv(mcButtonHeight, DialogUnits.Y, 8);
    ButtonSpacing := MulDiv(mcButtonSpacing, DialogUnits.X, 4) + 1;
    { Texh bounds }
    SetRect(TextRect, 0, 0, Screen.Width div 2 + MaxWidthInc, 0);
    DrawText(Canvas.Handle, PChar(AText), Length(AText) + 1, TextRect,
      DT_EXPANDTABS or DT_CALCRECT or DT_WORDBREAK);
    { Dialog icon }
    case Flags and $000000F0 of
      MB_ICONWARNING: IconID := IDI_WARNING;
      MB_ICONQUESTION: IconID := IDI_QUESTION;
      MB_ICONERROR: IconID := IDI_ERROR;
      MB_ICONINFORMATION: IconID := IDI_INFORMATION;
    else
      IconID := nil;
    end;
    IconTextWidth := TextRect.Right;
    IconTextHeight := TextRect.Bottom;
    if IconID <> nil then
    begin
      Inc(IconTextWidth, 32 + HorzSpacing);
      if IconTextHeight < 32 then IconTextHeight := 32;
    end;
    { Dialog bounds }
    ButtonCount := 0;
    for B := Low(TAppMsgBtn) to High(TAppMsgBtn) do
      if B in Buttons then Inc(ButtonCount);
    ButtonGroupWidth := 0;
    if ButtonCount <> 0 then
      ButtonGroupWidth := ButtonWidth * ButtonCount +
        ButtonSpacing * (ButtonCount - 1);
    ClientWidth := Max(IconTextWidth, ButtonGroupWidth) + HorzMargin * 2 - 5;
    ClientHeight := IconTextHeight + ButtonHeight + VertSpacing +
      VertMargin * 2 - 3;
    Left := (Screen.Width div 2) - (Width div 2);
    Top := (Screen.Height div 2) - (Height div 2);
    { Dialog caption }
    Caption := ACaption;
    { Create icon }
    if IconID <> nil then
      with TImage.Create(Result) do
      begin
        Name := 'Image';
        Parent := Result;
        Picture.Icon.Handle := LoadIcon(0, IconID);
        SetBounds(HorzMargin - 1, VertMargin - 2, 32, 32);
      end;
    { Create label (text) }
    with TLabel.Create(Result) do
    begin
      Name := 'Message';
      Parent := Result;
      WordWrap := True;
      Caption := AText;
      BoundsRect := TextRect;
      ALeft := IconTextWidth - TextRect.Right + HorzMargin;
      SetBounds(ALeft, 0, TextRect.Right, TextRect.Bottom);
      Top := VertMargin + Max(-2, (32 - TextRect.Bottom) div 2 - 1);
    end;
    { Create buttons }
    X := (ClientWidth - ButtonGroupWidth) div 2;
    ButtonCount := 0;
    for B := Low(TAppMsgBtn) to High(TAppMsgBtn) do
    begin
      if not (B in Buttons) then Continue;
      Inc(ButtonCount);
      { Create button }
      Button := TButton.Create(Result);
      with Button do
      begin
        Name := ButtonNames[B];
        Parent := Result;
        Caption := LoadResString(ButtonCaptions[B]);
        ModalResult := ModalResults[B];
        { Default button }
        case Flags and $00000F00 of
          MB_DEFBUTTON1: if ButtonCount = 1 then ActiveControl := Button;
          MB_DEFBUTTON2: if ButtonCount = 2 then ActiveControl := Button;
          MB_DEFBUTTON3: if ButtonCount = 3 then ActiveControl := Button;
          MB_DEFBUTTON4: if ButtonCount = 4 then ActiveControl := Button;
        end;
        { Cancel button }
        case Flags and $0000000F of
          MB_OKCANCEL: Cancel := B = mbCancel;
          MB_ABORTRETRYIGNORE: Cancel := False;
          MB_YESNOCANCEL: Cancel := B = mbCancel;
          MB_YESNO: Cancel := False;
          MB_RETRYCANCEL: Cancel := B = mbCancel;
        else
          Cancel := B = mbOK;
        end;
        { Button bounds }
        SetBounds(X, IconTextHeight + VertMargin + VertSpacing - 1,
          ButtonWidth, ButtonHeight);
        Inc(X, ButtonWidth + ButtonSpacing);
      end;
    end;
  end;
end;

function AppMessageBox(const Text, Caption: string; Flags: Longint): Integer;
begin
  with CreateAppMessageBox(Text, Caption, Flags) do
  try
    Result := ShowModal;
  finally
    Free;
  end;
end;

function AppMessageBoxFmt(const TextFormat, Caption: string; Flags: Longint; const Args: array of const): Integer;
begin
  Result := AppMessageBox(Format(TextFormat, Args), Caption, Flags);
end;

end.
