{
  POP3 Mail Checker

  Checker Options 

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit MC_Options;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  StdCtrls, Registry, ComCtrls, Math, Dialogs, ExtCtrls;

type

{ TOptions }

  TOptions = class(TObject)
  private
    FHost: string;
    FPort: string;
    FUserName: string;
    FPassword: string;
    FChecking: Boolean;
    FCheckInterval: Integer;
    FCheckOnStartup: Boolean;
    FTimeOut: Integer;
    FMessageBox: Boolean;
    FMessageBeep: Boolean;
    FShowReader: Boolean;
    FMailReader: string;
    FVisible: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure LoadFromRegistry;
    procedure SaveToRegistry;
    property Checking: Boolean read FChecking write FChecking;
    property CheckInterval: Integer read FCheckInterval write FCheckInterval;
    property CheckOnStartup: Boolean read FCheckOnStartup write FCheckOnStartup;
    property Host: string read FHost write FHost;
    property MailReader: string read FMailReader write FMailReader;
    property MessageBeep: Boolean read FMessageBeep write FMessageBeep;
    property MessageBox: Boolean read FMessageBox write FMessageBox;
    property Password: string read FPassword write FPassword;
    property Port: string read FPort write FPort;
    property ShowReader: Boolean read FShowReader write FShowReader;
    property TimeOut: Integer read FTimeOut write FTimeOut;
    property UserName: string read FUserName write FUserName;
    property Visible: Boolean read FVisible write FVisible;
  end;

{ TOptionsForm }

  TOptionsForm = class(TForm)
    ServerGroup: TGroupBox;
    HostLabel: TLabel;
    HostEdit: TEdit;
    UserLabel: TLabel;
    UserEdit: TEdit;
    PasswordLabel: TLabel;
    PasswordEdit: TEdit;
    ServerBevel: TBevel;
    PortLabel: TLabel;
    PortEdit: TEdit;
    TimeoutLabel: TLabel;
    TimeoutLabel2: TLabel;
    TimeoutTrackBar: TTrackBar;
    CheckGroup: TGroupBox;
    IntervalCheck: TCheckBox;
    IntervalEdit: TEdit;
    IntervalBar: TScrollBar;
    IntervalLabel: TLabel;
    StartupCheck: TCheckBox;
    MessageBoxCheck: TCheckBox;
    MessageBeepCheck: TCheckBox;
    ShowReaderCheck: TCheckBox;
    ReaderLabel: TLabel;
    ReaderEdit: TEdit;
    ReaderButton: TButton;
    ReaderDialog: TOpenDialog;
    OKButton: TButton;
    CancelButton: TButton;
    procedure IntervalCheckClick(Sender: TObject);
    procedure IntervalEditKeyPress(Sender: TObject; var Key: Char);
    procedure IntervalBarChange(Sender: TObject);
    procedure IntervalBarScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
    procedure ShowReaderCheckClick(Sender: TObject);
    procedure ReaderButtonClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure TimeoutTrackBarChange(Sender: TObject);
  private
    FOptions: TOptions;
    procedure CheckParams;
    procedure GetParams;
    procedure PutParams;
  public
    function Execute(Options: TOptions): Boolean;
  end;

function ChangeOptions(Options: TOptions): Boolean;

implementation

uses
  MC_MsgBox, MC_Consts, EncryptIt;

{$R *.DFM}

var
  FOptionsForm: TOptionsForm = nil;

function ChangeOptions(Options: TOptions): Boolean;
begin
  if FOptionsForm <> nil then
  begin
    FOptionsForm.BringToFront;
    Result := False;
    Exit;
  end;
  FOptionsForm := TOptionsForm.Create(Application);
  try
    Result := FOptionsForm.Execute(Options);
  finally
    FOptionsForm.Free;
    FOptionsForm := nil
  end;
end;

{ TOptions }

constructor TOptions.Create;
begin
  LoadFromRegistry;
end;

destructor TOptions.Destroy;
begin
  SaveToRegistry;
end;

procedure TOptions.LoadFromRegistry;
var
  IniFile: TRegIniFile;
begin
  IniFile := TRegIniFile.Create('Software\Mail Checker\1.0');
  try
    Host := IniFile.ReadString('', 'Host', '');
    Port := IniFile.ReadString('', 'Port', '110');
    UserName := IniFile.ReadString('', 'UserName', '');
    Password := Decrypt(IniFile.ReadString('', 'Password', ''), 3045);
    Checking := IniFile.ReadBool('', 'Checking', True);
    CheckInterval := IniFile.ReadInteger('', 'CheckInterval', 5);
    CheckOnStartup := IniFile.ReadBool('', 'CheckOnStartup', True);
    TimeOut := IniFile.ReadInteger('', 'TimeOut', 60);
    MessageBox := IniFile.ReadBool('', 'MessageBox', True);
    MessageBeep := IniFile.ReadBool('', 'MessageBeep', True);
    ShowReader := IniFile.ReadBool('', 'ShowReader', False);
    MailReader := IniFile.ReadString('', 'MailReader', '');
    Visible := IniFile.ReadBool('', 'Visible', True);
  finally
    IniFile.Free;
  end;
end;

procedure TOptions.SaveToRegistry;
var
  IniFile: TRegIniFile;
begin
  IniFile := TRegIniFile.Create('Software\Mail Checker\1.0');
  try
    IniFile.WriteString('', 'Host', Host);
    IniFile.WriteString('', 'Port', Port);
    IniFile.WriteString('', 'UserName', UserName);
    IniFile.WriteString('', 'Password', Encrypt(Password, 3045));
    IniFile.WriteBool('', 'Checking', Checking);
    IniFile.WriteInteger('', 'CheckInterval', CheckInterval);
    IniFile.WriteBool('', 'CheckOnStartup', CheckOnStartup);
    IniFile.WriteInteger('', 'TimeOut', TimeOut);
    IniFile.WriteBool('', 'MessageBox', MessageBox);
    IniFile.WriteBool('', 'MessageBeep', MessageBeep);
    IniFile.WriteBool('', 'Visible', Visible);
    IniFile.WriteBool('', 'ShowReader', ShowReader);
    IniFile.WriteString('', 'MailReader', MailReader);
  finally
    IniFile.Free;
  end;
end;

{ TOptionsForm }

procedure TOptionsForm.CheckParams;
const
  Flags = MB_ICONERROr or MB_OK;
begin
  { Check host name }
  with HostEdit do
  try
    if Length(Trim(Text)) = 0 then Abort;
  except
    AppMessageBox(SInvalidHost, Caption, Flags);
    SetFocus;
    Abort;
  end;
  { Check port number }
  with PortEdit do
  try
    StrToInt(Text);
  except
    AppMessageBox(SInvalidPort, Caption, Flags);
    SetFocus;
    Abort;
  end;
  { Check refresh interval }
  with IntervalEdit do
  try
    StrToInt(Text);
  except
    AppMessageBox(SInvalidInterval, Caption, Flags);
    SetFocus;
    Abort;
  end;
  { Check mail reader }
  with ReaderEdit do
  try
    if Enabled and (Length(Trim(Text)) = 0) then Abort;
  except
    AppMessageBox(SInvalidReader, Caption, Flags);
    SetFocus;
    Abort;
  end;
end;

procedure TOptionsForm.GetParams;
begin
  with FOptions do
  begin
    { Update controls }
    HostEdit.Text := Host;
    UserEdit.Text := UserName;
    PasswordEdit.Text := Password;
    PortEdit.Text := Port;
    TimeoutTrackBar.Position := MaxIntValue([1, MinIntValue([20, TimeOut div 15])]);
    TimeoutTrackBarChange(TimeoutTrackBar);
    IntervalCheck.Checked := Checking;
    IntervalEdit.Text := IntToStr(CheckInterval);
    IntervalBar.Position := 120 - CheckInterval;
    StartupCheck.Checked := CheckOnStartup;
    MessageBoxCheck.Checked := MessageBox;
    MessageBeepCheck.Checked := MessageBeep;
    ShowReaderCheck.Checked := ShowReader;
    ReaderEdit.Text := MailReader;
  end;
end;

procedure TOptionsForm.PutParams;
begin
  with FOptions do
  begin
    { Get values }
    Host := HostEdit.Text;
    UserName := UserEdit.Text;
    Password := PasswordEdit.Text;
    Port := PortEdit.Text;
    TimeOut := TimeoutTrackBar.Position * 15;
    Checking := IntervalCheck.Checked;
    CheckInterval := StrToIntDef(IntervalEdit.Text, CheckInterval);
    if CheckInterval < 1 then CheckInterval := 1;
    if CheckInterval > 120 then CheckInterval := 120;
    CheckOnStartup := StartupCheck.Checked;
    MessageBox := MessageBoxCheck.Checked;
    MessageBeep := MessageBeepCheck.Checked;
    ShowReader := ShowReaderCheck.Checked;
    MailReader := Trim(ReaderEdit.Text);
    { Store opeiotns }
    SaveToRegistry;
  end;
end;

function TOptionsForm.Execute(Options: TOptions): Boolean;
begin
  FOptions := Options;
  GetParams;
  Result := ShowModal = mrOK;
end;

procedure TOptionsForm.IntervalCheckClick(Sender: TObject);
begin
  if (Sender as TCheckBox).Checked then
  begin
    IntervalEdit.Color := clWindow;
    IntervalEdit.Enabled := True;
    IntervalBar.Enabled := True;
  end else
  begin
    IntervalEdit.Color := clBtnFace;
    IntervalEdit.Enabled := False;
    IntervalBar.Enabled := False;
  end;
end;

procedure TOptionsForm.IntervalEditKeyPress(Sender: TObject; var Key: Char);
begin
  if not (Key in ['0'..'9', #8]) then Key := #0;
end;

procedure TOptionsForm.TimeoutTrackBarChange(Sender: TObject);

  function MSecToStr(Time: Longint): string;
  var
    MSec, Sec, Min, Hour: Integer;
  begin
    { Decode time }
    Hour := Time div 3600000;
    Min := (Time - Hour * 3600000) div 60000;
    Sec := (Time - Hour * 3600000 - Min * 60000) div 1000;
    MSec := (Time - Hour * 3600000 - Min * 60000 - Sec * 1000) div 10;
    { Format result }
    if Sec > 0 then
      if (Time < 10000) and (MSec > 10) then
        Result := IntToStr(Sec) + DecimalSeparator + IntToStr(MSec) + ' ' + SShortSecStr
      else
        Result := IntToStr(Sec) + ' ' + SShortSecStr
    else
      Result := '';
    if Min > 0 then
      if Length(Result) > 0 then
        Result := IntToStr(Min) + ' ' + SShortMinStr + ' ' + Result
      else
        Result := IntToStr(Min) + ' ' + SShortMinStr;
    if Hour > 0 then
      if Length(Result) > 0 then
        Result := IntToStr(Hour) + ' ' + SShortHourStr + ' ' + Result
      else
        Result := IntToStr(Hour) + ' ' + SShortHourStr;
  end;

begin
  TimeoutLabel2.Caption := MSecToStr(TimeoutTrackBar.Position * 15 * 1000);
end;

procedure TOptionsForm.IntervalBarChange(Sender: TObject);
begin
  IntervalEdit.Text := IntToStr(120 - IntervalBar.Position);
end;

procedure TOptionsForm.IntervalBarScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  try
    ScrollPos := 120 - StrToInt(IntervalEdit.Text);
    case ScrollCode of
      scLineUp: Dec(ScrollPos, 1);
      scLineDown: Inc(ScrollPos, 1)
    end;
  except
    { Ignore exceptions }
  end;
end;

procedure TOptionsForm.ShowReaderCheckClick(Sender: TObject);
const
  Colors: array[Boolean] of TColor = (clBtnFace, clWindow);
begin
  with ShowReaderCheck do
  begin
    ReaderLabel.Enabled := Checked;
    ReaderEdit.Enabled := Checked;
    ReaderEdit.Color := Colors[Checked];
    ReaderButton.Enabled := Checked;
  end;
end;

procedure TOptionsForm.ReaderButtonClick(Sender: TObject);
begin
  with ReaderDialog do
  begin
    FileName := '';
    if Execute then ReaderEdit.Text := FileName;
  end;
end;

procedure TOptionsForm.OKButtonClick(Sender: TObject);
begin
  CheckParams;
  PutParams;
  ModalResult := mrOK;
end;

end.
