{
  POP3 Mail Checker

  POP3 session

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit MC_POP3;

interface

uses
  Classes, Pop3Prot, MC_Mail;

type
  TPOP3Session = class(TObject)
  private
    FPOP3Client: TSyncPop3Cli;
    FPOP3Buffer: TStringList;
    procedure POP3MessageBegin(Sender: TObject);
    procedure POP3MessageLine(Sender: TObject);
  public
    constructor Create;
    destructor Destroy; override;
    function Connect(const Host, Port: string; TimeOut: Integer): Boolean;
    function Quit: Boolean;
    function Login(const UserName, Password: string): Boolean;
    function ListMessages(var Count: Integer): Boolean;
    function ListMessageSize(Number: Integer; var Size: Integer): Boolean;
    function RetrieveHeader(Number: Integer; var Text: string): Boolean;
    function RetrieveMessage(Number: Integer; var Text: string): Boolean;
    function DeleteMessage(Number: Integer): Boolean;
  end;

implementation

{ TPOP3Session }

constructor TPOP3Session.Create;
begin
  FPOP3Client := TSyncPop3Cli.Create(nil);
  FPOP3Client.OnMessageBegin := POP3MessageBegin;
  FPOP3Client.OnMessageLine := POP3MessageLine;
  FPOP3Buffer := TStringList.Create;
end;

destructor TPOP3Session.Destroy;
begin
  FPOP3Buffer.Free;
  FPOP3Client.Free;
end;

procedure TPOP3Session.POP3MessageBegin(Sender: TObject);
begin
  FPOP3Buffer.Clear;
end;

procedure TPOP3Session.POP3MessageLine(Sender: TObject);
begin
  FPOP3Buffer.Add(FPOP3Client.LastResponse);
end;

function TPOP3Session.Connect(const Host, Port: string; TimeOut: Integer): Boolean;
begin
  try
    FPOP3Client.Host := Host;
    FPOP3Client.Port := Port;
    FPOP3Client.TimeOut := TimeOut;
    { Connect to server }
    Result := FPOP3Client.ConnectSync;
  except
    Result := False;
  end;
end;

function TPOP3Session.Quit: Boolean;
begin
  try
    Result := FPOP3Client.QuitSync;
  except
    Result := False;
  end;
end;

function TPOP3Session.Login(const UserName, Password: string): Boolean;
begin
  try
    FPOP3Client.UserName := UserName;
    FPOP3Client.Password := Password;
    { Logint to the server }
    Result := FPOP3Client.UserSync and FPOP3Client.PassSync;
  except
    Result := False;
  end;
end;

function TPOP3Session.ListMessages(var Count: Integer): Boolean;
begin
  try
    { Get message count }
    Result := FPOP3Client.StatSync;
    Count := FPOP3Client.MsgCount;
  except
    Result := False;
  end;
end;

function TPOP3Session.ListMessageSize(Number: Integer; var Size: Integer): Boolean;
begin
  try
    FPOP3Client.MsgNum := Number;
    { Get message size }
    Result := FPOP3Client.ListSync;
    Size := FPOP3Client.MsgSize;
  except
    Result := False;
  end;
end;

function TPOP3Session.RetrieveHeader(Number: Integer; var Text: string): Boolean;
begin
  try
    FPOP3Client.MsgNum := Number;
    { Get message header }
    Result := FPOP3Client.TopSync;
    Text := FPOP3Buffer.Text;
  except
    Result := False;
  end;
end;

function TPOP3Session.RetrieveMessage(Number: Integer; var Text: string): Boolean;
begin
  try
    FPOP3Client.MsgNum := Number;
    { Load message }
    Result := FPOP3Client.RetrSync;
    Text := FPOP3Buffer.Text;
  except
    Result := False;
  end;
end;

function TPOP3Session.DeleteMessage(Number: Integer): Boolean;
begin
  try
    FPOP3Client.MsgNum := Number;
    { Delete message }
    Result := FPOP3Client.DeleSync;
  except
    Result := False;
  end;
end;

end.
