{
  POP3 Mail Checker

  Message source form
                     
  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit MC_Source;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, Menus;

type
  TSourceForm = class(TForm)
    PopupMenu: TPopupMenu;
    PopupDecodeItem: TMenuItem;
    PopupBase64Item: TMenuItem;
    PopupQuotedPrintableItem: TMenuItem;
    PopupPlainTextItem: TMenuItem;
    PopupN1Item: TMenuItem;
    PopupCharSetItem: TMenuItem;
    PopupDOSItem: TMenuItem;
    PopupWINItem: TMenuItem;
    PopupKOIItem: TMenuItem;
    PopupN11Item: TMenuItem;
    PopupNoItem: TMenuItem;
    PopupN2Item: TMenuItem;
    PopupCopyItem: TMenuItem;
    PopupN3Item: TMenuItem;
    PopupSelectAllItem: TMenuItem;
    SourceMemo: TMemo;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure PopupMenuPopup(Sender: TObject);
    procedure PopupDecodeItemClick(Sender: TObject);
    procedure PopupCharSetItemClick(Sender: TObject);
    procedure PopupCopyItemClick(Sender: TObject);
    procedure PopupSelectAllItemClick(Sender: TObject);
    procedure SourceMemoKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    FMessage: string;
    FCharSet: Integer;
    FDecode: Integer;
    procedure SetCharSet(Value: Integer);
    procedure SetDecode(Value: Integer);
    procedure SetMessage(Value: string);
    procedure RefreshView;
  public
    { Public declarations }
  end;

procedure ShowMessageSource(const Message: string);

implementation

uses
  MC_Form, MC_Mail, MC_Decode, MC_CharSet, MIME;

{$R *.DFM}

procedure ShowMessageSource(const Message: string);
var
  I: Integer;
  F: TForm;
  S: string;
  SourceID: string;
  SourceForm: TSourceForm;
begin
  SourceID := ExtractMessageHeader(Message, SMessageID);
  { Find message source already opened }
  for I := 0 to Screen.FormCount - 1 do
  begin
    F := Screen.Forms[I];
    if F is TSourceForm then
    begin
      S := ExtractMessageHeader(TSourceForm(F).FMessage, SMessageID);
      if S = SourceID then
      begin
        F.BringToFront;
        Exit;
      end;
    end;
  end;
  { Open new message source form }
  SourceForm := TSourceForm.Create(Application);
  try
    SourceForm.SetCharSet(CheckerForm.CharSet);
    SourceForm.SetMessage(Message);
    SourceForm.Show;
  except
    SourceForm.Free;
    raise;
  end;
end;

{ TSourceForm }

procedure TSourceForm.SetCharSet(Value: Integer);
begin
  FCharSet := Value;
  RefreshView;
end;

procedure TSourceForm.SetDecode(Value: Integer);
begin
  FDecode := Value;
  RefreshView;
end;

procedure TSourceForm.SetMessage(Value: string);
begin
  FMessage := Value;
  RefreshView;
end;

procedure TSourceForm.RefreshView;
var
  Header, Body: string;
begin
  Header := ExtractMessageHeaders(FMessage);
  case FDecode of
    0: Body := ExtractMessageBody(FMessage);
    1: Body := StrDecodeQuotedPrintable(ExtractMessageBody(FMessage));
    2: Body := StrDecodeBase64(ExtractMessageBody(FMessage));
  end;
  Caption := ChangeCharSet(ExtractMessageHeader(FMessage, SSubject), FCharSet, CS_WIN);
  SourceMemo.Text := ChangeCharSet(Header + #13#10#13#10 + Body, FCharSet, CS_WIN);
end;

procedure TSourceForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TSourceForm.PopupMenuPopup(Sender: TObject);
begin
  PopupPlainTextItem.Checked := FDecode = 0;
  PopupQuotedPrintableItem.Checked := FDecode = 1;
  PopupBase64Item.Checked := FDecode = 2;
  PopupDOSItem.Checked := FCharSet = CS_DOS;
  PopupWINItem.Checked := FCharSet = CS_WIN;
  PopupKOIItem.Checked := FCharSet = CS_KOI;
  PopupNoItem.Checked := FCharSet = -1;
  PopupCopyItem.Enabled := SourceMemo.SelLength > 0;
end;

procedure TSourceForm.PopupDecodeItemClick(Sender: TObject);
begin
  SetDecode((Sender as TMenuItem).Tag)
end;

procedure TSourceForm.PopupCharSetItemClick(Sender: TObject);
begin
  SetCharSet((Sender as TMenuItem).Tag);
end;

procedure TSourceForm.PopupCopyItemClick(Sender: TObject);
begin
  SourceMemo.Perform(WM_COPY, 0, 0);
end;

procedure TSourceForm.PopupSelectAllItemClick(Sender: TObject);
begin
  SourceMemo.Perform(EM_SETSEL, 0, -1);
end;

procedure TSourceForm.SourceMemoKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssShift]) and (Key = VK_F10) then
  begin
    with SourceMemo.ClientOrigin do PopupMenu.Popup(X, Y);
    Key := 0;
  end;
end;

end.

