{
  POP3 Mail Checker

  TrayIcon component

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit MC_Tray;

interface

uses
  Windows, Messages, ShellAPI, ShlObj, SysUtils, Classes, Controls, Graphics,
  Forms, Menus;

type

{ TTrayIcon }

  TTrayPopupEvent = procedure(Sender: TObject; var AllowPopup: Boolean) of object;

  TTrayIcon = class(TComponent)
  private
    FHandle: HWnd;
    FData: TNotifyIconData;
    FVisible: Boolean;
    FIcon: TIcon;
    FIconID: Integer;
    FHint: string;
    FPopupMenu: TPopupMenu;
    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FOnPopup: TTrayPopupEvent;
    procedure IconChange(Sender: TObject);
    procedure SetIcon(Value: TIcon);
    procedure SetIconID(Value: Integer);
    procedure SetHint(Value: string);
    procedure SetPopupMenu(Value: TPopupMenu);
    procedure SetVisible(Value: Boolean);
    procedure WndProc(var Message: TMessage);
    procedure UpdateIcon;
  protected
    function AllowPopup: Boolean; dynamic;
    procedure Click; dynamic;
    procedure DblClick; dynamic;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Popup; dynamic;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Icon: TIcon read FIcon write SetIcon;
    property IconID: Integer read FIconID  write SetIconID default 0;
    property Hint: string read FHint write SetHint;
    property PopupMenu: TPopupMenu read FPopupMenu write SetPopupMenu;
    property Visible: Boolean read FVisible write SetVisible default False;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnPopup: TTrayPopupEvent read FOnPopup write FOnPopup;
  end;

implementation

{ TTrayIcon }

const
  WM_TRAYICON = WM_USER + 1;

constructor TTrayIcon.Create(AOwner: Tcomponent);
begin
  inherited Create(AOwner);
  FHandle := AllocateHWnd(WndProc);
  FIcon := TIcon.Create;
  FIcon.OnChange := IconChange;
end;

destructor TTrayIcon.Destroy;
begin
  SetVisible(False);
  DeallocateHWnd(FHandle);
  FIcon.Free;
  inherited Destroy;
end;

procedure TTrayIcon.IconChange(Sender: TObject);
begin
  UpdateIcon;
end;

procedure TTrayIcon.SetIcon(Value: TIcon);
begin
  FIcon.Assign(Value);
end;

procedure TTrayIcon.SetIconID(Value: Integer);
begin
  if FIconID <> Value then
  begin
    FIconID := Value;
    UpdateIcon;
  end;
end;

procedure TTrayIcon.SetHint(Value: string);
begin
  if FHint <> Value then
  begin
    FHint := Value;
    UpdateIcon;
  end;
end;

procedure TTrayIcon.SetPopupMenu(Value: TPopupMenu);
begin
  FPopupMenu := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

procedure TTrayIcon.SetVisible(Value: Boolean);
begin
  if FVisible <> Value then
  begin
    FVisible := Value;
    if not (csDesigning in ComponentState) then
      if Value then
        Shell_NotifyIcon(NIM_ADD, @FData)
      else
        Shell_NotifyIcon(NIM_DELETE, @FData);
  end;
end;

procedure TTrayIcon.UpdateIcon;
begin
  if not (csDesigning in ComponentState) then
  begin
    FillChar(FData, SizeOf(TNotifyIconData), 0);
    with FData do
    begin
      cbSize := SizeOf(TNotifyIconData);
      Wnd := FHandle;
      uID := FIconID;
      uFlags := NIF_MESSAGE or NIF_ICON or NIF_TIP;
      hIcon := FIcon.Handle;
      if Length(FHint) <> 0 then StrPLCopy(szTip, GetShortHint(FHint), SizeOf(szTip) - 1);
      uCallbackMessage := WM_TRAYICON;
    end;
    if Visible then Shell_NotifyIcon(NIM_MODIFY, @FData);
  end;
end;

procedure TTrayIcon.WndProc(var Message: TMessage);
begin
  with Message do
    if (Msg = WM_TRAYICON) and (wParam = FIconID) then
      case lParam of
        WM_LBUTTONUP: Click;
        WM_LBUTTONDBLCLK: DblClick;
        WM_RBUTTONUP: Popup;
      end
    else
      Result := DefWindowProc(FHandle, Msg, wParam, lParam);
end;

function TTrayIcon.AllowPopup: Boolean;
begin
  Result := True;
  if Assigned(FOnPopup) then FOnPopup(Self, Result);
end;

procedure TTrayIcon.Click;
begin
  if Assigned(FOnClick) then FOnClick(Self);
end;

procedure TTrayIcon.DblClick;
begin
  if Assigned(FOnDblClick) then FOnDblClick(Self);
end;

procedure TTrayIcon.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = PopupMenu) and (Operation = opRemove) then PopupMenu := nil;
end;

procedure TTrayIcon.Popup;
var
  P: TPoint;
begin
  if (PopupMenu <> nil) and PopupMenu.AutoPopup and AllowPopup then
  begin
    GetCursorPos(P);
    SetForegroundWindow(FHandle);
    PopupMenu.PopupComponent := Self;
    PopupMenu.Popup(P.X, P.Y);
  end;
end;

end.
